;****************************************************************
;
; TMS Demo - MSX ver 1.00 (C) Electric Adventures 2019
;
;****************************************************************
FNAME "MusicDemo.ROM"
cpu z80
ORG 4000h

INCLUDE "SVIRom-Include.asm"

PATSIZE:   EQU 73
SPRITECOUNT: EQU 1

    ; disable interrupts
    di
    ; set stack pointer
    ld sp,0ffffh
    ; interrupt mode 1 i.e just RST38 generated
    im 1
    ; set screen mode immediately, as interrupts are enabled on entry
    call SETSCREEN2
    ; jump to application start
    jp START
    ; remaining is free space before the RST38
    DS 44

ORG 0038h
RST_38:
    jp NMI

; This is the start of your application
START:
    CALL INITRAM

  ; initialise clock
  LD  HL,TIMER_TABLE
  LD  DE,TIMER_DATA_BLOCK
  CALL INIT_TIMER

  ;Enable timers - Tick, 1/4 sec and 1/2 sec timers
  CALL CREATE_TIMERS

  ; Seed random numbers with a fixed number (nothing else to use?)
  LD HL,1967
  CALL SEED_RANDOM


  ; Do all our VRAM setup
  ; NMI is currently disabled

TITLESCREEN:
  ; display our title screen
  DI
  ; Clear the screen
  CALL CLEARPAT

  ; Send the sprite definitions to the VDP
  LD HL,VRAM_SPRGEN
  LD DE,SPRITE_1
  LD BC,32*SPRITECOUNT
  CALL LDIRVM

  ; Clean up in case the game left anything on screen
  CALL CLEARSPRITES
  CALL SPRWRT

  ; Load the character set, make all three sections the same
  CALL LOAD_CHR_SET

  ; now setup our initial screen layout
  LD HL,VRAM_NAME
  LD DE,SL_DEMO
  LD BC,768
  CALL LDIRVM

  LD HL,VDU_WRITES
  CALL SET_VDU_HOOK
  EI

  LD A,12
  LD (CH1VOL),A

  CALL SET_ALL_SOUND_FREQUENCIES
  CALL SET_SOUND_VOLUME

  ; set cursor position
  LD A,68
  LD (SPRTBL),A
  LD A,56  
  LD (SPRTBL+1),A
  XOR A
  LD (SPRTBL+2),A
  LD A,8
  LD (SPRTBL+3),A

SPLASH_TITLE2:
  LD	A,(EighthSecTimer)
	CALL	TEST_SIGNAL
	OR	A
  JR Z,SPLASH_TITLE2
    ; Any other actions on the title screen go here

  CALL SELECT_CHANNEL
  CALL PLAYER_ACTIONS
  CALL SET_ALL_SOUND_FREQUENCIES
  CALL SET_SOUND_VOLUME
  
	JR SPLASH_TITLE2

PLAYER_ACTIONS:
  CALL JOYDIR
  BIT 0,A
  JR NZ,NUP
  LD A,(CURRENT_CHANNEL)
  JP INC_CHANNEL_VOL
NUP:
  BIT 3,A
  JR NZ,NRIGHT
  LD A,(CURRENT_CHANNEL)
  JP INC_CHANNEL_FREQ
NRIGHT:
  BIT 1,A
  JR NZ,NDOWN
  LD A,(CURRENT_CHANNEL)
  JP DEC_CHANNEL_VOL
NDOWN:
  BIT 2,A
  JR NZ,NLEFT
  LD A,(CURRENT_CHANNEL)
  JP DEC_CHANNEL_FREQ
NLEFT:
  RET

SELECT_CHANNEL:
  CALL JOYTST
  CP 0
  RET Z
  LD A,(CURRENT_CHANNEL)
  INC A
  CP 3
  JR C,.skip
    XOR A
.skip:
  LD (CURRENT_CHANNEL),A
  SLA A
  SLA A
  SLA A
  SLA A
  ADD A,68
  LD (SPRTBL),A
  RET

INC_CHANNEL_VOL:
  LD HL,CH1VOL
  CP 0
  JR Z,ICV1
ICV2:
  INC HL
  SUB 1
  JR NZ,ICV2
ICV1:
  LD A,(HL)
  ADD A,1
  AND %1111
  LD (HL),A
  RET

DEC_CHANNEL_VOL:
  LD HL,CH1VOL
  CP 0
  JR Z,DCV1
DCV2:
  INC HL
  SUB 1
  JR NZ,DCV2
DCV1:
  LD A,(HL)
  SUB 1
  AND %1111
  LD (HL),A
  RET

INC_CHANNEL_FREQ:
  LD HL,CH1FRQ
  CP 0
  JR Z,ICF1
ICF2:
  INC HL
  INC HL
  SUB 1
  JR NZ,ICF2
ICF1:
  LD A,(HL)
  ADD A,8
  LD (HL),A
  INC HL
  LD A,(HL)
  ADC A,0
  AND %11
  LD (HL),A
  RET

DEC_CHANNEL_FREQ:
  LD HL,CH1FRQ
  CP 0
  JR Z,DCF1
DCF2:
  INC HL
  INC HL
  SUB 1
  JR NZ,DCF2
DCF1:
  LD A,(HL)
  SUB 8
  LD (HL),A
  INC HL
  LD A,(HL)
  SBC A,0
  AND %11
  LD (HL),A
  RET

;**************************************************************
; Sound routines
;**************************************************************

; Initialise the sound chip, so that no sound is playing
INIT_SOUND:
  LD A,7        ; write to reg 7 mixer control
  LD E,0B8h     ; disable noise, enable all 3 tones
  CALL WRTPSG
  RET

; sets the current volume of each of the sound channels
SET_SOUND_VOLUME:
  LD HL,CH1VOL
  LD E,(HL) ; Channel 1 volume
  LD A,8
  CALL WRTPSG
  INC HL
  LD E,(HL) ; Channel 2 volume
  LD A,9
  CALL WRTPSG
  INC HL
  LD E,(HL) ; Channel 3 volume
  LD A,10
  CALL WRTPSG
  RET

SET_ALL_SOUND_FREQUENCIES:
  LD HL,CH1FRQ
  XOR A
  CALL SET_SOUND_FREQUENCY
  LD A,2
  CALL SET_SOUND_FREQUENCY
  LD A,4
  CALL SET_SOUND_FREQUENCY
  RET

SET_SOUND_FREQUENCY:
  LD E,(HL)
  CALL WRTPSG
  INC HL
  LD E,(HL)
  INC A
  CALL WRTPSG
  INC HL
  RET

;**************************************************************

; Load out character set from ROM, make all three sections the same
LOAD_CHR_SET:
    LD HL,0
SLOOP:
    LD DE,TILESET_1_PAT
    PUSH HL
    LD BC,PATSIZE*8
    CALL LDIRVM
    POP HL
    ; now load colour attributes
    PUSH HL
    LD BC,VRAM_COLOR
    ADD HL,BC
    LD DE,TILESET_1_COL
    LD BC,PATSIZE*8
    CALL LDIRVM
    POP HL
    LD BC,800h
    ADD HL,BC
    LD A,H
    CP 18h
    JR C,SLOOP
    RET

    RET

; Initialise any RAM we will be using
INITRAM:
  LD HL,0
  LD (CH1FRQ),HL
  LD (CH2FRQ),HL
  LD (CH3FRQ),HL
  XOR A
  LD (CHNCTL),A
  LD (CURRENT_CHANNEL),A
  LD A,0
  LD (CH1VOL),A
  LD (CH2VOL),A
  LD (CH3VOL),A
  RET

; This is our routine called every VDP interrupt during our main loop
; - Do all VDP writes here to avoid corruption
VDU_WRITES:
  LD HL,VRAM_NAME+17+32*9
  CALL SETWRT
  LD A,(CH1VOL)
  CALL bin2bcd
  LD (TEMP),A
  LD HL,TEMP
  CALL PRINTIT
  LD HL,VRAM_NAME+17+32*11
  CALL SETWRT
  LD A,(CH2VOL)
  CALL bin2bcd
  LD (TEMP),A
  LD HL,TEMP
  CALL PRINTIT
  LD HL,VRAM_NAME+17+32*13
  CALL SETWRT
  LD A,(CH3VOL)
  CALL bin2bcd
  LD (TEMP),A
  LD HL,TEMP
  CALL PRINTIT

  ; now output each channel frequency
  LD HL,VRAM_NAME+24+32*9
  CALL SETWRT
  LD HL,(CH1FRQ)
  CALL Bin162Bcd
  LD HL,BSD_OUTPUT+2
  CALL PRINTIT
  DEC HL
  CALL PRINTIT

  LD HL,VRAM_NAME+24+32*11
  CALL SETWRT
  LD HL,(CH2FRQ)
  CALL Bin162Bcd
  LD HL,BSD_OUTPUT+2
  CALL PRINTIT
  DEC HL
  CALL PRINTIT

  LD HL,VRAM_NAME+24+32*13
  CALL SETWRT
  LD HL,(CH3FRQ)
  CALL Bin162Bcd
  LD HL,BSD_OUTPUT+2
  CALL PRINTIT
  DEC HL
  CALL PRINTIT
  RET

; This is our routine called every VDP interrupt during the title screen
; - Do all VDP writes here to avoid corruption
OUTPUT_VDP_TITLE:
    RET

; output characters to the name table
; Video Ram write position already should be set
; HL = Pointer to text buffer to write, end at FFh
OUTPUT_TEXT:
    LD A,(HL)
    CP 255
    RET Z
    OUT (DATA_PORT),A
    INC HL
    JR OUTPUT_TEXT

; Title text
TITLE_TEXT:
db 30,23,29,0,14,41,49,51,63,255

;**************************************************************************************************
; Include external data files
;**************************************************************************************************

include "SVIRom-MusicDemo-Patterns.ASM"


;**************************************************************************************************
; Standard Libraries
;**************************************************************************************************

include "SVIRom-Lib.ASM"

END:    EQU $

;**************************************************************************************************
; RAM Definitions
;**************************************************************************************************

ORG RAMSTART

CURRENT_CHANNEL: DS 1
TEMP: DS 1

; Sound variables
CH1FRQ: DS 2
CH2FRQ: DS 2
CH3FRQ: DS 2

CHNCTL: DS 1

CH1VOL: DS 1
CH2VOL: DS 1
CH3VOL: DS 1

